

(function()
{
  var lang = 'en'; // default

  var langIsSet = false;
  
  /**
   * A function to change the currently used language dictionary
   * 
   * @param newLang   Sets the new language
   * @return          String of the now-current language (not always the same as newLang)
   **/
  
  window.setLanguage = function(newLang)
  {
    var oldLang = lang;
    
    lang = (newLang || navigator.language).toLowerCase();

    while (true)
    {
      for (i=0; i<languages.length; i++)
      {
        if (languages[i]==lang)
        {
          break;
        }
      }
      if ( i<languages.length )
      {
        // insert the script then remove it
        var ele = document.createElement('script');
        ele.setAttribute('type','text/javascript');
        ele.setAttribute('src', 'lang/'+lang+'.js');
        (document.getElementsByTagName('head')[0] || document.documentElement).appendChild(ele);
        ele.parentNode.removeChild(ele); // cleanup
        break;
      }
      if (lang.indexOf('-')+1)
      {
        lang = lang.substr(0,lang.indexOf('-'));
      }
      else
      {
        if (lang==oldLang)
        {
          throw Error('Cannot set language');
        }
        lang = oldLang;
      }
    }
    langIsSet = true;
    return lang;
  }

  /**
   * This prototype function takes a string as the 'this' value and uses that
   * as the key for the translation.
   *
   * The 'replace' argument is optional and is overloaded.
   * - If 'replace' is a string then it will replace the substring %s before the
   *   return value is returned
   * - If 'replace' is an object then the key-value pairs will replace the substrings
   *   in the form of %(key)s
   *   So for example the replace of {name:"Ben",city:"Oslo"} will modify "Hello %(name)s in %(city)s."
   *   to become "Hello Ben in Oslo".
   *
   * @param replace   An optional string or object
   * @return          The translated string with the replacements made
   **/

  String.prototype.toLocale = function(replace)
  {
    if (!langIsSet) { window.setLanguage() }

    var str;
    if (window.localeStrings && (str = localeStrings[this]) !==undefined)
    {
      if (typeof replace == 'string')
      {
        str = str.replace("%s",replace);
      }
      else
      {
        
        if (typeof replace == 'object')
        {
          str = str.replace(
            /%\(([^)]*)\)s/g,
            function(m0, m1)
            {
              return ''+(replace.hasOwnProperty(m1) ? replace[m1] : m0);
            }
          );
        }
      }
      return str;
    }
    else
    {
      throw Error("Localised string not found: "+this);
    }
  };

  /**
   * Translates all children elements, applies to all Elements and document
   *
   * @param newLang   If present then the language will change to this (if possible)
   * @return          The string of the current language
   **/


  Element.prototype.toLocale = document.toLocale = function(newLang)
  {
    if (newLang) { window.setLanguage(newLang) }
  
    var
      ele = this.getElementsByTagName('*'),
      result = null,
      key = null
    ;
    
    for (var i=0, e; e = ele[i]; i++)  // loop every element
    {
      result = e.getAttribute('locale');
      if (result) // yes, element is translatable
      {
        result = result.replace(/^\s+|\s+$/g,'').split(/\s*;+\s*/g);
        for (j=0; j<result.length; j++)
        {
          if (result)
          {
            // apply translation for each attribute
            key = result[j].split(/\s*:\s*/);

            if (key[1])
            {
              e[ key[0] ] = key[1].toLocale();
            }
            else
            {
              e.innerText = key[0].toLocale();
            }
          }
        }
      }
    }
    return lang;
  };
})();